@@ -129,14 +129,6 @@ def OnDropItemToPC ():
 
 	#-3 : drop stuff in inventory (but not equippable slots)
 	GemRB.DropDraggedItem (pc, -3)
-	if GemRB.IsDraggingItem ()==1:
-		if GUICommon.HasTOB ():
-			GemRB.DisplayString (61794, 0xffffff)
-		elif GUICommon.GameIsPST ():
-			GemRB.DisplayString (19257, 0xffffff)
-		else:
-			GemRB.DisplayString (17999, 0xffffff)
-
 	GUIINV.UpdateInventoryWindow ()
 	return
 
@@ -307,7 +299,7 @@ def DisplayItem (itemresref, type):
 	Button = Window.GetControl(9)
 	drink = (type&1) and (item["Function"]&1)
 	read = (type&1) and (item["Function"]&2)
-	#sorcerors cannot learn spells
+	# sorcerers cannot learn spells
 	# FIXME: unhardcode
 	pc = GemRB.GameGetSelectedPCSingle ()
 	if GemRB.GetPlayerStat (pc, IE_CLASS) == 19:
@@ -331,6 +323,9 @@ def DisplayItem (itemresref, type):
 			# FIXME: where do mods add the new string? This is untranslatable
 			Button.SetText ("Open container")
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenItemWindow)
+		# Just skip the redundant info page and go directly to the container
+		if GemRB.GetVar("GUIEnhancements")&GE_ALWAYS_OPEN_CONTAINER_ITEMS:
+			OpenItemWindow()
 	elif dialog:
 		Button.SetText (strrefs[5])
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DialogItemWindow)
@@ -363,13 +358,18 @@ def DisplayItem (itemresref, type):
 	ItemInfoWindow.ShowModal(MODAL_SHADOW_GRAY)
 	return
 
-def OpenItemInfoWindow ():
+def OpenItemInfoWindow (slot = None):
 	pc = GemRB.GameGetSelectedPCSingle ()
-	if GUICommon.GameIsPST():
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+
+	if slot == None:
+		if GUICommon.GameIsPST():
+			slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+		else:
+			slot = GemRB.GetVar ("ItemButton")
+			slot_item = GemRB.GetSlotItem (pc, slot)
 	else:
-		slot = GemRB.GetVar ("ItemButton")
 		slot_item = GemRB.GetSlotItem (pc, slot)
+
 	item = GemRB.GetItem (slot_item["ItemResRef"])
 
 	if TryAutoIdentification(pc, item, slot, slot_item, True):
@@ -698,7 +698,7 @@ def CloseErrorWindow ():
 	return
 
 def ReadItemWindow ():
-	global level, spell_ref, pause
+	global pause
 	"""Tries to learn the mage scroll."""
 
 	pc = GemRB.GameGetSelectedPCSingle ()
@@ -707,60 +707,44 @@ def ReadItemWindow ():
 
 	if ret:
 		# couldn't find any strrefs for the other undhandled values (stat, level)
-		strref = 72873
-		CloseItemInfoWindow ()
-		if GUICommon.HasTOB() and ret == LSR_KNOWN:
-			OpenErrorWindow (strref)
-		return
-
-	if GUICommon.GameIsPST():
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
-	else:
-		slot_item = GemRB.GetSlotItem (pc, slot)
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
-	spell = GemRB.GetSpell (spell_ref)
-	if spell:
-		# can we learn more spells of this level?
-		level = spell['SpellLevel']-1
-		spell_count = GemRB.GetKnownSpellsCount (pc, IE_SPELL_TYPE_WIZARD, level)
-		if spell_count > GemRB.GetAbilityBonus (IE_INT, 2, GemRB.GetPlayerStat (pc, IE_INT)):
-			ret = LSR_FULL
-			if GUICommon.GameIsBG2():
-				strref = 32097
-			else:
-				strref = -1
+		if ret == LSR_KNOWN and GUICommon.HasTOB():
+			strref = 72873
+		elif ret == LSR_KNOWN and GUICommon.GameIsPST():
+			strref = 50394
+		elif ret == LSR_FULL and GUICommon.GameIsBG2():
+			strref = 32097
+		elif ret == LSR_FULL and GUICommon.GameIsPST():
+			strref = 50395
+		elif GUICommon.GameIsPST():
+			strref = 4250
 		else:
-			pause = GemRB.GamePause(3,1) #query the pause state
-			GemRB.GamePause(0,1)
-			GemRB.UseItem (pc, slot, 1, 5)
-			GemRB.SetTimedEvent(DelayedReadItemWindow, 1)
-			return
-	else:
-		print "WARNING: invalid spell header in item", slot_item['ItemResRef']
-		CloseItemInfoWindow ()
-		return -1
+			strref = 10831
 
-	CloseItemInfoWindow ()
-	OpenErrorWindow (strref)
+		CloseItemInfoWindow ()
+		OpenErrorWindow (strref)
+
+	else:
+		if GUICommon.GameIsPST():
+			slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+
+		pause = GemRB.GamePause(3,1) #query the pause state
+		GemRB.GamePause(0,1)
+		GemRB.UseItem (pc, slot, 1, 5)
+		GemRB.SetTimedEvent(DelayedReadItemWindow, 1)
 
 	return ret
 
 def DelayedReadItemWindow ():
-	global level, spell_ref
-
 	#restore the pause state
 	if pause:
 		GemRB.GamePause(1,1)
 
-	pc = GemRB.GameGetSelectedPCSingle ()
-	if Spellbook.HasSpell (pc, IE_SPELL_TYPE_WIZARD, level, spell_ref)!=-1:
-		strref = 10830
-	else:
-		ret = LSR_FAILED
-		strref = 10831
 	CloseItemInfoWindow ()
-	if not GUICommon.GameIsPST():
-		OpenErrorWindow (strref)
+	if GUICommon.GameIsPST():
+		strref = 4249
+	else:
+		strref = 10830
+	OpenErrorWindow (strref)
 	return
 
 def OpenItemWindow ():
@@ -811,7 +795,7 @@ def IdentifyUseSpell ():
 		ItemInfoWindow.Unload ()
 	GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
 	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow()
+	OpenItemInfoWindow(slot)
 	return
 
 def IdentifyUseScroll ():
@@ -828,7 +812,7 @@ def IdentifyUseScroll ():
 	if GemRB.HasSpecialItem (pc, 1, 1):
 		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
 	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow()
+	OpenItemInfoWindow(slot)
 	return
 
 def CloseIdentifyItemWindow ():
