@@ -363,8 +363,7 @@ Effect *EffectQueue::CreateEffect(ieDword opcode, ieDword param1, ieDword param2
 	memset(fx,0,sizeof(Effect));
 	fx->Target = FX_TARGET_SELF;
 	fx->Opcode = opcode;
-	//probability2 is the low number (by effectqueue 331)
-	fx->Probability1 = 100;
+	fx->ProbabilityRangeMax = 100;
 	fx->Parameter1 = param1;
 	fx->Parameter2 = param2;
 	fx->TimingMode = timing;
@@ -456,9 +455,11 @@ Effect *EffectQueue::CreateUnsummonEffect(Effect *fx)
 	if( (fx->TimingMode&0xff) == FX_DURATION_INSTANT_LIMITED) {
 		newfx = CreateEffectCopy(fx, fx_unsummon_creature_ref, 0, 0);
 		newfx->TimingMode = FX_DURATION_DELAY_PERMANENT;
+		newfx->Target = FX_TARGET_PRESET;
 		if( newfx->Resource3[0]) {
 			strnuprcpy(newfx->Resource,newfx->Resource3, sizeof(ieResRef)-1 );
 		} else {
+			//FIXME: unhardcode; should probably use a spell hit (shtable) in some way
 			strnuprcpy(newfx->Resource,"SPGFLSH1", sizeof(ieResRef)-1 );
 		}
 		if( fx->TimingMode == FX_DURATION_ABSOLUTE) {
@@ -811,10 +812,8 @@ static inline bool check_level(Actor *target, Effect *fx)
 //roll should hit in the middle
 static inline bool check_probability(Effect* fx)
 {
-	//watch for this, probability1 is the high number
-	//probability2 is the low number
 	//random value is 0-99
-	if(fx->random_value<fx->Probability2 || fx->random_value>fx->Probability1) {
+	if (fx->random_value<fx->ProbabilityRangeMin || fx->random_value>fx->ProbabilityRangeMax) {
 		return false;
 	}
 	return true;
@@ -1055,12 +1054,17 @@ static bool check_resistance(Actor* actor, Effect* fx)
 	if (iwd2fx) {
 		// 3ed style check
 		// TODO: check if luck really affects it (i doubt it does)
-		ieDword check = fx->CasterLevel + actor->LuckyRoll(1, 20, 0);
+		int roll = actor->LuckyRoll(1, 20, 0);
+		ieDword check = fx->CasterLevel + roll;
+		int penetration = 0;
 		// +2/+4 level bonus from the (greater) spell penetration feat
 		if (caster && caster->HasFeat(FEAT_SPELL_PENETRATION)) {
-			check += 2 * caster->GetStat(IE_FEAT_SPELL_PENETRATION);
+			penetration += 2 * caster->GetStat(IE_FEAT_SPELL_PENETRATION);
 		}
+		check += penetration;
 		resisted = (signed) check < (signed) val;
+		// ~Spell Resistance check (Spell resistance:) %d vs. (d20 + caster level + spell resistance mod)  = %d + %d + %d.~
+		displaymsg->DisplayRollStringName(39673, DMC_LIGHTGREY, actor, val, roll, fx->CasterLevel, penetration);
 	} else {
 		// 2.5 style check
 		resisted = (signed) fx->random_value < (signed) val;
@@ -1113,6 +1117,9 @@ static bool check_resistance(Actor* actor, Effect* fx)
 int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieDword resistance) const
 {
 	//print("FX 0x%02x: %s(%d, %d)", fx->Opcode, effectnames[fx->Opcode].Name, fx->Parameter1, fx->Parameter2);
+	if (fx->TimingMode == FX_DURATION_JUST_EXPIRED) {
+		return FX_NOT_APPLIED;
+	}
 	if( fx->Opcode >= MAX_EFFECTS) {
 		fx->TimingMode = FX_DURATION_JUST_EXPIRED;
 		return FX_NOT_APPLIED;
@@ -1128,9 +1135,9 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieD
 		}
 
 		//gemrb specific, stat based chance
-		if ((fx->Probability2 == 100) && Owner && (Owner->Type==ST_ACTOR) ) {
-			fx->Probability2 = 0;
-			fx->Probability1 = ((Actor *) Owner)->GetSafeStat(fx->Probability1);
+		if ((fx->ProbabilityRangeMin == 100) && Owner && (Owner->Type==ST_ACTOR) ) {
+			fx->ProbabilityRangeMin = 0;
+			fx->ProbabilityRangeMax = ((Actor *) Owner)->GetSafeStat(fx->ProbabilityRangeMax);
 		}
 
 		if (resistance) {
@@ -1792,57 +1799,34 @@ void EffectQueue::AddWeaponEffects(EffectQueue *fxqueue, EffectRef &fx_ref) cons
 	}
 }
 
-/* no longer needed, use IE_CASTING stat
-static EffectRef fx_disable_spellcasting_ref = { "DisableCasting", -1 };
-int EffectQueue::DisabledSpellcasting(int types) const
+// figure out how much damage reduction applies for a given weapon enchantment and damage type
+int EffectQueue::SumDamageReduction(EffectRef &effect_reference, ieDword weaponEnchantment, int &total) const
 {
-	ResolveEffectRef(fx_disable_spellcasting_ref);
-	if( fx_disable_spellcasting_ref.opcode < 0) {
-		return 0;
-	}
+	ResolveEffectRef(effect_reference);
+	ieDword opcode = effect_reference.opcode;
+	int remaining = 0;
+	int count = 0;
 
-	unsigned int spelltype_mask = 0;
-	ieDword opcode = fx_disable_spellcasting_ref.opcode;
 	std::list< Effect* >::const_iterator f;
-	for ( f = effects.begin(); f != effects.end(); f++ ) {
+	for (f = effects.begin(); f != effects.end(); f++) {
 		MATCH_OPCODE();
 		MATCH_LIVE_FX();
 
-		if (iwd2fx) {
-			switch((*f)->Parameter2) {
-				case 0: // all
-					spelltype_mask |= 7;
-					break;
-				case 1: // mage and cleric
-					spelltype_mask |= 3;
-					break;
-				case 2: // mage
-					spelltype_mask |= 2;
-					break;
-				case 3: // cleric
-					spelltype_mask |= 1;
-					break;
-				case 4: // innate
-					spelltype_mask |= 4;
-					break;
-			}
-		} else {
-			switch((*f)->Parameter2) {
-				case 0: // mage
-					spelltype_mask |= 2;
-					break;
-				case 1: // cleric
-					spelltype_mask |= 1;
-					break;
-				case 2: // innate
-					spelltype_mask |= 4;
-					break;
-			}
+		Effect* fx = *f;
+		if (!fx) continue;
+		count++;
+		// add up if the effect has enough enchantment (or ignores it)
+		if (!fx->Parameter2 || fx->Parameter2 > weaponEnchantment) {
+			remaining += fx->Parameter1;
 		}
+		total += fx->Parameter1;
+	}
+	if (count) {
+		return remaining;
+	} else {
+		return -1;
 	}
-	return spelltype_mask & types;
 }
-*/
 
 //useful for immunity vs spell, can't use item, etc.
 Effect *EffectQueue::HasOpcodeWithResource(ieDword opcode, const ieResRef resource) const
