@@ -25,7 +25,9 @@
 #include "win32def.h"
 #include "ie_cursors.h"
 
+#include "Game.h"
 #include "Interface.h"
+#include "KeyMap.h"
 #include "Video.h"
 #include "GUI/Window.h"
 //N900
@@ -131,7 +133,8 @@ void EventMgr::Clear()
 /** Remove a Window from the array */
 void EventMgr::DelWindow(Window *win)
 {
-	if (last_win_focused == win) {
+	bool focused = (last_win_focused == win);
+	if (focused) {
 		last_win_focused = NULL;
 	}
 	if (last_win_mousefocused == win) {
@@ -158,6 +161,10 @@ void EventMgr::DelWindow(Window *win)
 			for (t = topwin.begin(); t != topwin.end(); ++t) {
 				if ( (*t) == pos) {
 					topwin.erase( t );
+					if (focused && topwin.size() > 0) {
+						//revert focus to new top window
+						SetFocused(windows[topwin[0]], NULL);
+					}
 					return;
 				}
 			}
@@ -351,9 +358,19 @@ void EventMgr::KeyPress(unsigned char Key, unsigned short Mod)
 	//
 	if (last_win_focused == NULL) return;
 	Control *ctrl = last_win_focused->GetFocus();
-	if (ctrl == NULL) return;
-	ctrl->OnKeyPress( Key, Mod );
+	if (!ctrl || !ctrl->OnKeyPress( Key, Mod )) {
+		// FIXME: need a better way to determine when to call ResolveKey/SetHotKey
+		if (core->GetGameControl()
+			&& !MButtons // checking for drag actions
+			&& !core->IsPresentingModalWindow()
+			&& !core->GetKeyMap()->ResolveKey(Key, 0)) {
+			core->GetGame()->SetHotKey(toupper(Key));
+		}
+		//this is to refresh changing mouse cursors should the focus change)
+		FakeMouseMove();
+	}
 }
+
 /** BroadCast Key Release Event */
 void EventMgr::KeyRelease(unsigned char Key, unsigned short Mod)
 {
@@ -362,11 +379,11 @@ void EventMgr::KeyRelease(unsigned char Key, unsigned short Mod)
 		fakeRMB=false;
 	//
 	if (last_win_focused == NULL) return;
-	Control *ctrl = last_win_focused->GetFocus();
 	if (Key == GEM_GRAB) {
 		core->GetVideoDriver()->ToggleGrabInput();
 		return;
 	}
+	Control *ctrl = last_win_focused->GetFocus();
 	if (ctrl == NULL) return;
 	ctrl->OnKeyRelease( Key, Mod );
 }
@@ -435,30 +452,27 @@ void EventMgr::OnSpecialKeyPress(unsigned char Key)
 					return;
 				}
 				break;
-			case IE_GUI_GAMECONTROL:
-			case IE_GUI_WORLDMAP:
-				//gamecontrols will receive all special keys
-				break;
-			case IE_GUI_EDIT:
-			case IE_GUI_TEXTAREA:
-				//editboxes and textareas will receive all special keys
-				break;
-			default:
-				//other controls don't receive any
-				return;
+				// shouldnt be any harm in sending these events to any control
 		}
 		ctrl->OnSpecialKeyPress( Key );
 	}
 }
 
+/** Trigger a fake MouseMove event with current coordinates (typically used to
+ *  refresh the cursor without actual user input) */
+void EventMgr::FakeMouseMove()
+{
+	int x, y;
+	core->GetVideoDriver()->GetMousePos(x, y);
+	MouseMove((unsigned short) x, (unsigned short) y);
+}
+
 void EventMgr::SetFocused(Window *win, Control *ctrl)
 {
 	last_win_focused = win;
 	last_win_focused->SetFocused(ctrl);
 	//this is to refresh changing mouse cursors should the focus change)
-	int x,y;
-	core->GetVideoDriver()->GetMousePos(x,y);
-	MouseMove((unsigned short) x, (unsigned short) y);
+	FakeMouseMove();
 }
 
 void EventMgr::SetDCDelay(unsigned long t)
