@@ -24,7 +24,6 @@
 
 #include "win32def.h"
 
-#include "Audio.h"
 #include "DialogHandler.h"
 #include "GameData.h"
 #include "ImageMgr.h"
@@ -54,7 +53,6 @@ TextArea::TextArea(Color hitextcolor, Color initcolor, Color lowtextcolor)
 	seltext = -1;
 	Value = 0xffffffff;
 	ResetEventHandler( TextAreaOnChange );
-	ResetEventHandler( TextAreaOutOfText );
 	PortraitResRef[0]=0;
 	palette = core->CreatePalette( hitextcolor, lowtextcolor );
 	initpalette = core->CreatePalette( initcolor, lowtextcolor );
@@ -151,10 +149,11 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 	if (XPos == 65535) {
 		return;
 	}
-	size_t linesize = lines.size();
-	if (linesize == 0) {
+
+	if (lines.size() == 0) {
 		return;
 	}
+	size_t linesize = lines.size() - 1; // -1 because 0 counts
 
 	//if textarea is 'selectable' it actually means, it is a listbox
 	//in this case the selected value equals the line number
@@ -166,7 +165,7 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 		Buffer[0] = 0;
 		int len = 0;
 		int lastlen = 0;
-		for (size_t i = 0; i < linesize; i++) {
+		for (size_t i = 0; i <= linesize; i++) {
 			if (strnicmp( "[s=", lines[i], 3 ) == 0) {
 				int tlen;
 				unsigned long acolor, bcolor;
@@ -198,7 +197,7 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 				memcpy( &Buffer[lastlen], lines[i], len - lastlen );
 			}
 			lastlen = len;
-			if (i != linesize - 1) {
+			if (i != linesize) {
 				Buffer[lastlen - 1] = '\n';
 				Buffer[lastlen] = 0;
 			}
@@ -218,29 +217,19 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 		short LineOffset = (short)(TextYPos % ftext->maxHeight);
 		Region textClip(clip.x, clip.y - LineOffset, clip.w, clip.h + LineOffset);
 
-
 		ftext->PrintFromLine( startrow, textClip,
 							 ( unsigned char * ) Buffer, palette,
 							 IE_FONT_ALIGN_LEFT, finit, Cursor, pos );
 		free( Buffer );
 		video->SetClipRect( NULL );
-		//streaming text
-		if (linesize>50) {
-			//the buffer is filled enough
-			return;
+
+		if ((Flags &IE_GUI_TEXTAREA_SMOOTHSCROLL)
+			&& linesize <= (size_t)(1 + (Height - 1) / ftext->maxHeight)) {
+			// streaming text: keep pushing newlines until we have an entire areas worth of them.
+			// this way it will appear that the text scrolls out of view
+			AppendText("\n", -1);
 		}
-		if (core->GetAudioDrv()->IsSpeaking() ) {
-			//the narrator is still talking
-			return;
-		}
-		if (RunEventHandler( TextAreaOutOfText )) {
-			return;
-		}
-		if (linesize==lines.size()) {
-			ResetEventHandler( TextAreaOutOfText );
-			return;
-		}
-		AppendText("\n",-1);
+
 		return;
 	}
 	// normal scrolling textarea
@@ -472,7 +461,7 @@ void TextArea::SetFonts(Font* init, Font* text)
 }
 
 /** Key Press Event */
-void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
+bool TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 {
 	if (Flags & IE_GUI_TEXTAREA_EDITABLE) {
 		if (Key >= 0x20) {
@@ -491,24 +480,24 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			CalcRowCount();
 			RunEventHandler( TextAreaOnChange );
 		}
-		return;
+		return true;
 	}
 
 	//Selectable=false for dialogs, rather unintuitive, but fact
 	if ((Flags & IE_GUI_TEXTAREA_SELECTABLE) || ( Key < '1' ) || ( Key > '9' ))
-		return;
+		return false;
 	GameControl *gc = core->GetGameControl();
 	if (gc && (gc->GetDialogueFlags()&DF_IN_DIALOG) ) {
 		Changed = true;
 		seltext=minrow-1;
 		if ((unsigned int) seltext>=lines.size()) {
-			return;
+			return true;
 		}
 		for(int i=0;i<Key-'0';i++) {
 			do {
 				seltext++;
 				if ((unsigned int) seltext>=lines.size()) {
-					return;
+					return true;
 				}
 			}
 			while (strnicmp( lines[seltext], "[s=", 3 ) != 0 );
@@ -518,20 +507,22 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 		if (idx==-1) {
 			//this kills this object, don't use any more data!
 			gc->dialoghandler->EndDialog();
-			return;
+			return true;
 		}
 		gc->dialoghandler->DialogChoose( idx );
+		return true;
 	}
+	return false;
 }
 
 /** Special Key Press */
-void TextArea::OnSpecialKeyPress(unsigned char Key)
+bool TextArea::OnSpecialKeyPress(unsigned char Key)
 {
 	int len;
 	int i;
 
 	if (!(Flags&IE_GUI_TEXTAREA_EDITABLE)) {
-		return;
+		return false;
 	}
 	Owner->Invalidate();
 	Changed = true;
@@ -643,6 +634,7 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 	}
 	CalcRowCount();
 	RunEventHandler( TextAreaOnChange );
+	return true;
 }
 
 /** Returns Row count */
@@ -734,12 +726,11 @@ void TextArea::CalcRowCount()
 		for (size_t i = 0; i < lines.size(); i++) {
 //			rows++;
 			tr = 0;
-			int len = ( int ) strlen( lines[i] );
-			char* tmp = (char *) malloc( len + 1 );
-			memcpy( tmp, lines[i], len + 1 );
+			ieWord* tmp = NULL;
+			size_t len = (int)ftext->GetDoubleByteString((unsigned char*)lines[i], tmp);
 			ftext->SetupString( tmp, w );
-			for (int p = 0; p <= len; p++) {
-				if (( ( unsigned char ) tmp[p] ) == '[') {
+			for (size_t p = 0; p <= len; p++) {
+				if (( tmp[p] ) == '[') {
 					p++;
 					//char tag[256];
 					int k = 0;
@@ -828,10 +819,13 @@ void TextArea::OnMouseOver(unsigned short /*x*/, unsigned short y)
 }
 
 /** Mouse Button Up */
-void TextArea::OnMouseUp(unsigned short x, unsigned short y, unsigned short /*Button*/,
+void TextArea::OnMouseUp(unsigned short x, unsigned short y, unsigned short Button,
 	unsigned short /*Mod*/)
 {
-	if (( x <= Width ) && ( y <= ( Height - 5 ) ) && ( seltext != -1 )) {
+	if (!(Button & (GEM_MB_ACTION|GEM_MB_MENU)))
+		return;
+
+	if ((x < Width) && (y < Height - 5) && (seltext != -1)) {
 		Value = (unsigned int) seltext;
 		Changed = true;
 		if (strnicmp( lines[seltext], "[s=", 3 ) == 0) {
@@ -879,7 +873,7 @@ void TextArea::CopyTo(TextArea *ta)
 	ta->SetText(lines);
 }
 
-void TextArea::RedrawTextArea(const char* VariableName, unsigned int Sum)
+void TextArea::UpdateState(const char* VariableName, unsigned int Sum)
 {
 	if (strnicmp( VarName, VariableName, MAX_VARIABLE_LENGTH )) {
 		return;
@@ -900,7 +894,7 @@ void TextArea::SelectText(const char *select)
 			} else {
 				SetRow( i );
 			}
-			RedrawTextArea( VarName, i);
+			UpdateState(VarName, i);
 			CalcRowCount();
 			Owner->Invalidate();
 			core->RedrawAll();
@@ -925,9 +919,6 @@ bool TextArea::SetEvent(int eventType, EventHandler handler)
 	case IE_GUI_TEXTAREA_ON_CHANGE:
 		TextAreaOnChange = handler;
 		break;
-	case IE_GUI_TEXTAREA_OUT_OF_TEXT:
-		TextAreaOutOfText = handler;
-		break;
 	default:
 		return false;
 	}
@@ -975,29 +966,18 @@ void TextArea::SetupScroll()
 	SetPreservedRow(0);
 	startrow = 0;
 	// ticks is the number of ticks it takes to scroll this font 1 px
-	ticks = 2000 / ftext->maxHeight; // 2000 is just a value based off the previous magic number.
+	ticks = 2400 / ftext->maxHeight;
 	//clearing the textarea
 	Clear();
-	unsigned int i = (unsigned int) (Height/ftext->maxHeight);
+	unsigned int i = (unsigned int) (1 + ((Height - 1) / ftext->maxHeight)); // ceiling
 	while (i--) { //push empty lines so that the text starts out of view.
 		char *str = (char *) malloc(1);
 		str[0]=0;
 		lines.push_back(str);
 		lrows.push_back(0);
 	}
-	i = (unsigned int) lines.size();
 	Flags |= IE_GUI_TEXTAREA_SMOOTHSCROLL;
 	starttime = GetTickCount();
-	if (RunEventHandler( TextAreaOutOfText )) {
-		//event handler destructed this object?
-		return;
-	}
-	if (i==lines.size()) {
-		ResetEventHandler( TextAreaOutOfText );
-		return;
-	}
-	//recalculates rows
-	AppendText("\n",-1);
 }
 
 void TextArea::OnMouseDown(unsigned short /*x*/, unsigned short /*y*/, unsigned short Button,
