@@ -25,6 +25,7 @@
 #include "defsounds.h"
 #include "ie_feats.h"
 
+#include "AmbientMgr.h"
 #include "Audio.h"
 #include "CharAnimations.h"
 #include "DialogHandler.h"
@@ -136,8 +137,8 @@ int GetReaction(Actor *target, Scriptable *Sender)
 	reaction = 10 + rmodrep[rep] + rmodchr[chr];
 
 	// add -4 penalty when dealing with racial enemies
-	if (Sender && target->GetRangerLevel() && Sender->Type == ST_ACTOR && target->IsRacialEnemy((Actor *)Sender)) {
-		reaction -= 4;
+	if (Sender && target->GetRangerLevel() && (Sender->Type == ST_ACTOR) ) {
+		reaction -= target->GetRacialEnemyBonus((Actor *) Sender);
 	}
 
 	return reaction;
@@ -1160,6 +1161,7 @@ bool CreateMovementEffect(Actor* actor, const char *area, const Point &position)
 	fx->SetPosition(position);
 	strnuprcpy(fx->Resource, area, 8);
 	core->ApplyEffect(fx, actor, actor);
+	delete fx;
 	return true;
 }
 
@@ -1286,21 +1288,27 @@ void AttackCore(Scriptable *Sender, Scriptable *target, int flags)
 	assert(Sender && Sender->Type == ST_ACTOR);
 	Actor *actor = (Actor *) Sender;
 
-	WeaponInfo wi;
-	ITMExtHeader *header = NULL;
-	ITMExtHeader *hittingheader = NULL;
-	int tohit;
-	int DamageBonus, CriticalBonus;
-	int speed, style;
+	// if held or disabled, etc, then cannot start or continue attacking
+	if (actor->Immobile()) {
+		actor->roundTime = 0;
+		Sender->ReleaseCurrentAction();
+		return;
+	}
+
+	// mislead and projected images can't attack
+	int puppet = actor->GetStat(IE_PUPPETMASTERTYPE);
+	if (puppet && puppet < 3) {
+		Log(DEBUG, "AttackCore", "Tried attacking with an illusionary copy: %s!", actor->GetName(1));
+		return;
+	}
 
-	bool leftorright = false;
 	Actor *tar = NULL;
 	if (target->Type==ST_ACTOR) {
 		tar = (Actor *) target;
 
 		// release if target is invisible to sender (because of death or invisbility spell)
 		if (tar->IsInvisibleTo(Sender) || (tar->GetSafeStat(IE_STATE_ID) & STATE_DEAD)){
-			actor->SetStance(IE_ANI_READY);
+			actor->StopAttack();
 			Sender->ReleaseCurrentAction();
 			actor->AddTrigger(TriggerEntry(trigger_targetunreachable, tar->GetGlobalID()));
 			Log(WARNING, "AttackCore", "Tried attacking invisible/dead actor: %s!", tar->GetName(1));
@@ -1314,20 +1322,22 @@ void AttackCore(Scriptable *Sender, Scriptable *target, int flags)
 		return;
 	}
 
+	WeaponInfo wi;
+	bool leftorright = false;
+	ITMExtHeader *header = actor->GetWeapon(wi, leftorright && actor->IsDualWielding());
 	//will return false on any errors (eg, unusable weapon)
-	if (!actor->GetCombatDetails(tohit, leftorright, wi, header, hittingheader, DamageBonus, speed, CriticalBonus, style, tar)) {
-		actor->SetStance(IE_ANI_READY);
+	if (!header || !actor->WeaponIsUsable(leftorright, header)) {
+		actor->StopAttack();
 		Sender->ReleaseCurrentAction();
 		actor->AddTrigger(TriggerEntry(trigger_unusable, tar->GetGlobalID()));
 		Log(WARNING, "AttackCore", "Weapon unusable: %s!", actor->GetName(1));
 		return;
 	}
 
-	if (header) wi.range *= 10;
-	else wi.range = 0;
+	unsigned int weaponrange = actor->GetWeaponRange(wi);
 
 	if ( target->Type == ST_DOOR || target->Type == ST_CONTAINER) {
-		wi.range += 10;
+		weaponrange += 10;
 	}
 	if (!(flags&AC_NO_SOUND) ) {
 		if (!Sender->CurrentActionTicks) {
@@ -1337,16 +1347,17 @@ void AttackCore(Scriptable *Sender, Scriptable *target, int flags)
 				actor->VerbalConstant(VB_ATTACK, 5);
 			}
 			//display attack message
-			displaymsg->DisplayConstantStringAction(STR_ACTION_ATTACK, DMC_WHITE, Sender, target);
+			if (target->GetGlobalID() != Sender->LastTarget) {
+				displaymsg->DisplayConstantStringAction(STR_ACTION_ATTACK, DMC_WHITE, Sender, target);
+			}
 		}
 	}
-	//action performed
-	actor->FaceTarget( target );
 
 	if ( Sender->GetCurrentArea()!=target->GetCurrentArea() ||
-		(PersonalDistance(Sender, target) > wi.range) ||
-		(!Sender->GetCurrentArea()->IsVisibleLOS(Sender->Pos, target->Pos))) {
-		MoveNearerTo(Sender, target, wi.range);
+		(PersonalDistance(Sender, target) > weaponrange) ||
+		(!Sender->GetCurrentArea()->IsVisibleLOS(Sender->Pos, target->Pos)) ||
+		!CanSee(Sender, target, true, 0)) {
+		MoveNearerTo(Sender, target, weaponrange);
 		return;
 	} else if (target->Type == ST_DOOR) {
 		//Forcing a lock does not launch the trap...
@@ -1364,6 +1375,8 @@ void AttackCore(Scriptable *Sender, Scriptable *target, int flags)
 		Sender->ReleaseCurrentAction();
 		return;
 	}
+	//action performed
+	actor->FaceTarget(target);
 
 	Sender->LastTarget = target->GetGlobalID();
 	actor->PerformAttack(core->GetGame()->GameTime);
@@ -1465,6 +1478,7 @@ static void ParseObject(const char *&str,const char *&src, Object *&object)
 			}
 			src++; //skipping (
 			if (*src==')') {
+				src++;
 				break;
 			}
 			Nesting++;
@@ -1699,7 +1713,7 @@ void GoNear(Scriptable *Sender, const Point &p)
 	Sender->AddActionInFront( action );
 }
 
-void MoveNearerTo(Scriptable *Sender, Scriptable *target, int distance)
+void MoveNearerTo(Scriptable *Sender, Scriptable *target, int distance, int dont_release)
 {
 	Point p;
 	Map *myarea, *hisarea;
@@ -1737,7 +1751,7 @@ void MoveNearerTo(Scriptable *Sender, Scriptable *target, int distance)
 		distance += ((Actor *)target)->size*10;
 	}
 
-	MoveNearerTo(Sender, p, distance, 0);
+	MoveNearerTo(Sender, p, distance, dont_release);
 }
 
 //It is not always good to release the current action if target is unreachable
@@ -1945,7 +1959,7 @@ Trigger *GenerateTriggerCore(const char *src, const char *str, int trIndex, int
 				else {
 					i=0;
 				}
-				while (*src != '"') {
+				while (*src != '"' && *src != '#') {
 					if (*src == 0) {
 						delete newTrigger;
 						return NULL;
@@ -1981,7 +1995,7 @@ Trigger *GenerateTriggerCore(const char *src, const char *str, int trIndex, int
 					}
 					//reading the context string
 					i=0;
-					while (*src != '"') {
+					while (*src != '"' && *src != '#') {
 						if (*src == 0) {
 							delete newTrigger;
 							return NULL;
@@ -2014,8 +2028,7 @@ void SetVariable(Scriptable* Sender, const char* VarName, const char* Context, i
 			VarName, value );
 	}
 
-	strncpy( newVarName, Context, 6 );
-	newVarName[6]=0;
+	strlcpy( newVarName, Context, 7 );
 	if (strnicmp( newVarName, "MYAREA", 6 ) == 0) {
 		Sender->GetCurrentArea()->locals->SetAt( VarName, value, NoCreate );
 		return;
@@ -2059,22 +2072,21 @@ void SetVariable(Scriptable* Sender, const char* VarName, ieDword value)
 	if (InDebug&ID_VARIABLES) {
 		Log(DEBUG, "GSUtils", "Setting variable(\"%s\", %d)", VarName, value );
 	}
-	strncpy( newVarName, VarName, 6 );
-	newVarName[6]=0;
-	if (strnicmp( newVarName, "MYAREA", 6 ) == 0) {
+	strlcpy( newVarName, VarName, 7 );
+	if (stricmp( newVarName, "MYAREA" ) == 0) {
 		Sender->GetCurrentArea()->locals->SetAt( poi, value, NoCreate );
 		return;
 	}
-	if (strnicmp( newVarName, "LOCALS", 6 ) == 0) {
+	if (stricmp( newVarName, "LOCALS" ) == 0) {
 		Sender->locals->SetAt( poi, value, NoCreate );
 		return;
 	}
 	Game *game = core->GetGame();
-	if (HasKaputz && !strnicmp(newVarName,"KAPUTZ",6) ) {
+	if (HasKaputz && !stricmp(newVarName,"KAPUTZ") ) {
 		game->kaputz->SetAt( poi, value, NoCreate );
 		return;
 	}
-	if (strnicmp(newVarName,"GLOBAL",6) ) {
+	if (stricmp(newVarName,"GLOBAL") ) {
 		Map *map=game->GetMap(game->FindMap(newVarName));
 		if (map) {
 			map->locals->SetAt( poi, value, NoCreate);
@@ -2095,22 +2107,21 @@ ieDword CheckVariable(Scriptable* Sender, const char* VarName, bool *valid)
 	const char *poi;
 	ieDword value = 0;
 
-	strncpy( newVarName, VarName, 6 );
-	newVarName[6]=0;
+	strlcpy( newVarName, VarName, 7 );
 	poi = &VarName[6];
 	//some HoW triggers use a : to separate the scope from the variable name
 	if (*poi==':') {
 		poi++;
 	}
 
-	if (strnicmp( newVarName, "MYAREA", 6 ) == 0) {
+	if (stricmp( newVarName, "MYAREA" ) == 0) {
 		Sender->GetCurrentArea()->locals->Lookup( poi, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s: %d", VarName, value);
 		}
 		return value;
 	}
-	if (strnicmp( newVarName, "LOCALS", 6 ) == 0) {
+	if (stricmp( newVarName, "LOCALS" ) == 0) {
 		Sender->locals->Lookup( poi, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s: %d", VarName, value);
@@ -2118,14 +2129,14 @@ ieDword CheckVariable(Scriptable* Sender, const char* VarName, bool *valid)
 		return value;
 	}
 	Game *game = core->GetGame();
-	if (HasKaputz && !strnicmp(newVarName,"KAPUTZ",6) ) {
+	if (HasKaputz && !stricmp(newVarName,"KAPUTZ") ) {
 		game->kaputz->Lookup( poi, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s: %d", VarName, value);
 		}
 		return value;
 	}
-	if (strnicmp(newVarName,"GLOBAL",6) ) {
+	if (stricmp(newVarName,"GLOBAL") ) {
 		Map *map=game->GetMap(game->FindMap(newVarName));
 		if (map) {
 			map->locals->Lookup( poi, value);
@@ -2152,16 +2163,15 @@ ieDword CheckVariable(Scriptable* Sender, const char* VarName, const char* Conte
 	char newVarName[8];
 	ieDword value = 0;
 
-	strncpy(newVarName, Context, 6);
-	newVarName[6]=0;
-	if (strnicmp( newVarName, "MYAREA", 6 ) == 0) {
+	strlcpy(newVarName, Context, 7);
+	if (stricmp( newVarName, "MYAREA" ) == 0) {
 		Sender->GetCurrentArea()->locals->Lookup( VarName, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s%s: %d", Context, VarName, value);
 		}
 		return value;
 	}
-	if (strnicmp( newVarName, "LOCALS", 6 ) == 0) {
+	if (stricmp( newVarName, "LOCALS" ) == 0) {
 		Sender->locals->Lookup( VarName, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s%s: %d", Context, VarName, value);
@@ -2169,14 +2179,14 @@ ieDword CheckVariable(Scriptable* Sender, const char* VarName, const char* Conte
 		return value;
 	}
 	Game *game = core->GetGame();
-	if (HasKaputz && !strnicmp(newVarName,"KAPUTZ",6) ) {
+	if (HasKaputz && !stricmp(newVarName,"KAPUTZ") ) {
 		game->kaputz->Lookup( VarName, value );
 		if (InDebug&ID_VARIABLES) {
 			print("CheckVariable %s%s: %d", Context, VarName, value);
 		}
 		return value;
 	}
-	if (strnicmp(newVarName,"GLOBAL",6) ) {
+	if (stricmp(newVarName,"GLOBAL") ) {
 		Map *map=game->GetMap(game->FindMap(newVarName));
 		if (map) {
 			map->locals->Lookup( VarName, value);
@@ -2198,6 +2208,33 @@ ieDword CheckVariable(Scriptable* Sender, const char* VarName, const char* Conte
 	return value;
 }
 
+// checks if a variable exists in any context
+bool VariableExists(Scriptable *Sender, const char *VarName, const char *Context)
+{
+	ieDword value = 0;
+	char newVarName[8];
+	strlcpy(newVarName, Context, 7);
+	Game *game = core->GetGame();
+
+	if (Sender->GetCurrentArea()->locals->Lookup(VarName, value)) {
+		return true;
+	} else if (Sender->locals->Lookup(VarName, value)) {
+		return true;
+	} else if (HasKaputz && game->kaputz->Lookup(VarName, value)) {
+		return true;
+	} else if (game->locals->Lookup(VarName, value)) {
+		return true;
+	} else {
+		Map *map = game->GetMap(game->FindMap(newVarName));
+		if (map) {
+			if (map->locals->Lookup(VarName, value)) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
 int DiffCore(ieDword a, ieDword b, int diffmode)
 {
 	switch (diffmode) {
@@ -2388,7 +2425,7 @@ unsigned int GetSpellDistance(const ieResRef spellres, Scriptable *Sender)
 	}
 
 	gamedata->FreeSpell(spl, spellres, false);
-	return dist*8; //FIXME: empirical constant to convert from points to (feet)
+	return dist*9; //FIXME: empirical constant to convert from points to (feet)
 }
 
 /* returns an item's casting distance, it depends on the used header, and targeting mode too
@@ -2477,8 +2514,13 @@ void AmbientActivateCore(Scriptable *Sender, Action *parameters, int flag)
 		anim = Sender->GetCurrentArea( )->GetAnimation( parameters->objects[1]->objectName );
 	}
 	if (!anim) {
-		Log(ERROR, "GSUtils", "No Animation Named \"%s\" or \"%s\"",
-			parameters->string0Parameter,parameters->objects[1]->objectName );
+		// iwd2 expects this behaviour in ar6001 by (de)activating sound_portal
+		AmbientMgr *ambientmgr = core->GetAudioDrv()->GetAmbientMgr();
+		if (flag) {
+			ambientmgr->activate(parameters->objects[1]->objectName);
+		} else {
+			ambientmgr->deactivate(parameters->objects[1]->objectName);
+		}
 		return;
 	}
 	int i;
