@@ -33,6 +33,7 @@
 #include "Cache.h"
 #include "Callback.h"
 #include "Holder.h"
+#include "InterfaceConfig.h"
 
 #include <map>
 #include <string>
@@ -117,6 +118,7 @@ struct DamageInfoStruct {
 	unsigned int resist_stat;
 	unsigned int value;
 	int iwd_mod_type;
+	int reduction;
 	// maybe also add the ac bonus and/or the DL_ constants
 };
 
@@ -150,13 +152,6 @@ struct SurgeSpell {
 	ieStrRef message;
 };
 
-// whitelist of config values we can push directly into vars
-struct INIWhiteListEntry {
-	const char* INITag;
-	const char* INIKey;
-	int defaultValue;
-};
-
 class ItemList {
 public:
 	ieResRef *ResRefs;
@@ -347,7 +342,7 @@ private:
 	ieResRef Palette32;
 	ieResRef Palette256;
 	ieDword* slotmatrix; //itemtype vs slottype
-	ieDword* itemtypedata; //armor failure, critical multiplier
+	std::vector<std::vector<int> > itemtypedata; //armor failure, critical multiplier, critical range
 	SlotType* slottypes;
 	int ItemTypes;
 	int tooltip_x;
@@ -376,6 +371,7 @@ private:
 	KeyMap *keymap;
 	std::string Encoding;
 public:
+	std::string TLKEncoding;
 	Holder<StringMgr> strings;
 	GlobalTimer * timer;
 	Palette *InfoTextPalette;
@@ -404,9 +400,9 @@ public:
 	TimeStruct Time;
 	std::vector<SurgeSpell> SurgeSpells;
 public:
-	Interface(int iargc, char *iargv[]);
+	Interface();
 	~Interface(void);
-	int Init(void);
+	int Init(InterfaceConfig* config);
 	//TODO: Core Methods in Interface Class
 	void SetFeature(int value, int position);
 	/* don't rely on the exact return value of this function */
@@ -444,6 +440,7 @@ public:
 	/** Loads a Window in the Window Manager */
 	int LoadWindow(unsigned short WindowID);
 	/** Creates a Window in the Window Manager */
+	bool IsPresentingModalWindow() {return (bool)(ModalWindow);};
 #ifdef WIN32
 #ifdef CreateWindow
 #undef CreateWindow
@@ -569,6 +566,8 @@ public:
 	/** load saved game by index (-1 is default), ver_override is an optional parameter
 	to override the saved game's version */
 	void LoadGame(SaveGame *save, int ver_override);
+	/** reloads the world map from a resource file */
+	void UpdateWorldMap(ieResRef wmResRef);
 	/** fix changes in global script/worldmap*/
 	void UpdateMasterScript();
 	/*reads the filenames of the portraits folder into a list */
@@ -587,8 +586,11 @@ public:
 	ieDword QuerySlotEffects(unsigned int idx) const;
 	const char * QuerySlotResRef(unsigned int idx) const;
 	int GetArmorFailure(unsigned int itemtype) const;
+	int GetShieldFailure(unsigned int itemtype) const;
+	int GetArmorPenalty(unsigned int itemtype) const;
 	int GetShieldPenalty(unsigned int itemtype) const;
 	int GetCriticalMultiplier(unsigned int itemtype) const;
+	int GetCriticalRange(unsigned int itemtype) const;
 	/*returns true if an itemtype is acceptable for a slottype, also checks the usability flags */
 	int CanUseItemType(int slottype, Item *item, Actor *actor=NULL, bool feedback=false, bool equipped=false) const;
 	/*removes single file from cache*/
@@ -725,8 +727,6 @@ public:
 	bool SaveConfig();
 private:
 	int LoadSprites();
-	bool LoadConfig(void);
-	bool LoadConfig(const char *filename);
 	bool LoadGemRBINI();
 	/** Load the encoding table selected in gemrb.cfg */
 	bool LoadEncoding();
@@ -760,7 +760,7 @@ private:
 	ieDword *GetListFrom2DAInternal(const ieResRef resref);
 public:
 	//N900 specific config
-        char RMBK;
+	char RMBK[_MAX_PATH];
         int ProximitySensorEnabled;
         //End N900 variables
 	char GameDataPath[_MAX_PATH];
@@ -770,12 +770,10 @@ public:
 	char GamePortraitsPath[_MAX_PATH];
 	char GameCharactersPath[_MAX_PATH];
 	char GemRBOverridePath[_MAX_PATH];
+	char GemRBUnhardcodedPath[_MAX_PATH];
 	ieResRef GameNameResRef;
 	ieResRef GoldResRef; //MISC07.itm
 	Variables *RtRows;
-	char UserDir[_MAX_PATH];
-	int argc;
-	char **argv;
 	char CustomFontPath[_MAX_PATH];
 	char GameName[_MAX_PATH];
 	char GameType[_MAX_PATH];
@@ -842,10 +840,6 @@ public:
 	Audio* GetAudioDrv(void) const;
 
 	void SetTickHook(EventHandler);
-
-#ifdef _DEBUG
-	int FileStreamPtrCount;
-#endif
 };
 
 extern GEM_EXPORT Interface * core;
