@@ -172,8 +172,8 @@ void Container::DestroyContainer()
 CREItem *Container::RemoveItem(unsigned int idx, unsigned int count)
 {
 	CREItem *ret = inventory.RemoveItem(idx, count);
-	//we just took the 3. or less item, groundpile changed
-	if ((Type == IE_CONTAINER_PILE) && (inventory.GetSlotCount()<3)) {
+	//if we just took one of the first few items, groundpile changed
+	if ((Type == IE_CONTAINER_PILE) && (idx<MAX_GROUND_ICON_DRAWN)) {
 		RefreshGroundIcons();
 	}
 	return ret;
@@ -185,7 +185,7 @@ int Container::AddItem(CREItem *item)
 {
 	inventory.AddItem(item);
 	//we just added a 3. or less item, groundpile changed
-	if ((Type == IE_CONTAINER_PILE) && (inventory.GetSlotCount()<4)) {
+	if ((Type == IE_CONTAINER_PILE) && (inventory.GetSlotCount()<=MAX_GROUND_ICON_DRAWN)) {
 		RefreshGroundIcons();
 	}
 	return 2;
@@ -237,13 +237,21 @@ void Container::TryPickLock(Actor *actor)
 		}
 		return;
 	}
-	if (actor->GetStat(IE_LOCKPICKING)<LockDifficulty) {
+	int stat = actor->GetStat(IE_LOCKPICKING);
+	if (core->HasFeature(GF_3ED_RULES)) {
+		stat *= 7; // convert to percent (magic 7 is from RE)
+		int dexmod = actor->GetAbilityBonus(IE_DEX);
+		stat += dexmod; // the original didn't use it, so let's not multiply it
+		displaymsg->DisplayRollStringName(39301, DMC_LIGHTGREY, actor, stat-dexmod, LockDifficulty, dexmod);
+	}
+	if (stat < LockDifficulty) {
 		displaymsg->DisplayConstantStringName(STR_LOCKPICK_FAILED, DMC_BG2XPGREEN, actor);
 		AddTrigger(TriggerEntry(trigger_picklockfailed, actor->GetGlobalID()));
 		core->PlaySound(DS_PICKFAIL); //AMB_D21
 		return;
 	}
 	SetContainerLocked(false);
+	core->GetGameControl()->ResetTargetMode();
 	displaymsg->DisplayConstantStringName(STR_LOCKPICK_DONE, DMC_LIGHTGREY, actor);
 	AddTrigger(TriggerEntry(trigger_unlocked, actor->GetGlobalID()));
 	core->PlaySound(DS_PICKLOCK); //AMB_D21D
@@ -267,6 +275,13 @@ void Container::TryBashLock(Actor *actor)
 	}
 	unsigned int roll = actor->LuckyRoll(1, 10, bonus, 0);
 
+	if (core->HasFeature(GF_3ED_RULES)) {
+		// ~Bash door check. Roll %d + %d Str mod > %d door DC.~
+		// there is no separate string for non-doors
+		displaymsg->DisplayRollStringName(20460, DMC_LIGHTGREY, actor, roll, bonus, LockDifficulty);
+	}
+
+	actor->FaceTarget(this);
 	if(roll < LockDifficulty || LockDifficulty == 100) {
 		displaymsg->DisplayConstantStringName(STR_CONTBASH_FAIL, DMC_BG2XPGREEN, actor);
 		return;
@@ -274,6 +289,7 @@ void Container::TryBashLock(Actor *actor)
 
 	displaymsg->DisplayConstantStringName(STR_CONTBASH_DONE, DMC_LIGHTGREY, actor);
 	SetContainerLocked(false);
+	core->GetGameControl()->ResetTargetMode();
 	//Is this really useful ?
 	AddTrigger(TriggerEntry(trigger_unlocked, actor->GetGlobalID()));
 	ImmediateEvent();
