@@ -112,8 +112,7 @@ void Door::UpdateDoor()
 
 	if (Flags & DOOR_TRANSPARENT) {
 		pmdflags = PATH_MAP_DOOR_IMPASSABLE;
-	}
-	else {
+	} else {
 		//both door flags are needed here, one for transparency the other
 		//is for passability
 		pmdflags = PATH_MAP_DOOR_OPAQUE|PATH_MAP_DOOR_IMPASSABLE;
@@ -338,16 +337,38 @@ void Highlightable::TryDisarm(Actor *actor)
 	if (!Trapped || !TrapDetected) return;
 
 	int skill = actor->GetStat(IE_TRAPS);
+	int roll = 0;
+	int bonus = 0;
+	int trapDC = TrapRemovalDiff;
 
-	if (skill/2+core->Roll(1,skill/2,0)>TrapRemovalDiff) {
+	if (core->HasFeature(GF_3ED_RULES)) {
+		roll = core->Roll(1, 20, 0);
+		bonus = actor->GetAbilityBonus(IE_INT);
+		trapDC = TrapRemovalDiff/7 + 10; // oddity from the original
+	} else {
+		roll = core->Roll(1, skill/2, 0);
+		skill /= 2;
+	}
+
+	int check = skill + roll + bonus;
+	if (check > trapDC) {
 		AddTrigger(TriggerEntry(trigger_disarmed, actor->GetGlobalID()));
 		//trap removed
 		Trapped = 0;
+		if (core->HasFeature(GF_3ED_RULES)) {
+			// ~Successful Disarm Device - d20 roll %d + Disarm Device skill %d + INT mod %d >= Trap DC %d~
+			displaymsg->DisplayRollStringName(39266, DMC_LIGHTGREY, actor, roll, skill-bonus, bonus, trapDC);
+		}
 		displaymsg->DisplayConstantStringName(STR_DISARM_DONE, DMC_LIGHTGREY, actor);
 		int xp = actor->CalculateExperience(XP_DISARM, actor->GetXPLevel(1));
 		Game *game = core->GetGame();
 		game->ShareXP(xp, SX_DIVIDE);
+		core->GetGameControl()->ResetTargetMode();
 	} else {
+		if (core->HasFeature(GF_3ED_RULES)) {
+			// ~Failed Disarm Device - d20 roll %d + Disarm Device skill %d + INT mod %d >= Trap DC %d~
+			displaymsg->DisplayRollStringName(39266, DMC_LIGHTGREY, actor, roll, skill-bonus, bonus, trapDC);
+		}
 		displaymsg->DisplayConstantStringName(STR_DISARM_FAIL, DMC_LIGHTGREY, actor);
 		TriggerTrap(skill, actor->GetGlobalID());
 	}
@@ -364,13 +385,21 @@ void Door::TryPickLock(Actor *actor)
 		}
 		return;
 	}
-	if (actor->GetStat(IE_LOCKPICKING)<LockDifficulty) {
+	int stat = actor->GetStat(IE_LOCKPICKING);
+	if (core->HasFeature(GF_3ED_RULES)) {
+		stat *= 7; // convert to percent (magic 7 is from RE)
+		int dexmod = actor->GetAbilityBonus(IE_DEX);
+		stat += dexmod; // the original didn't use it, so let's not multiply it
+		displaymsg->DisplayRollStringName(39301, DMC_LIGHTGREY, actor, stat-dexmod, LockDifficulty, dexmod);
+	}
+	if (stat < (signed)LockDifficulty) {
 		displaymsg->DisplayConstantStringName(STR_LOCKPICK_FAILED, DMC_BG2XPGREEN, actor);
 		AddTrigger(TriggerEntry(trigger_picklockfailed, actor->GetGlobalID()));
 		core->PlaySound(DS_PICKFAIL);
 		return;
 	}
 	SetDoorLocked( false, true);
+	core->GetGameControl()->ResetTargetMode();
 	displaymsg->DisplayConstantStringName(STR_LOCKPICK_DONE, DMC_LIGHTGREY, actor);
 	AddTrigger(TriggerEntry(trigger_unlocked, actor->GetGlobalID()));
 	core->PlaySound(DS_PICKLOCK);
@@ -394,6 +423,12 @@ void Door::TryBashLock(Actor *actor)
 	}
 	unsigned int roll = actor->LuckyRoll(1, 10, bonus, 0);
 
+	actor->FaceTarget(this);
+	if (core->HasFeature(GF_3ED_RULES)) {
+		// ~Bash door check. Roll %d + %d Str mod > %d door DC.~
+		displaymsg->DisplayRollStringName(20460, DMC_LIGHTGREY, actor, roll, bonus, LockDifficulty);
+	}
+
 	if(roll < LockDifficulty || LockDifficulty == 100) {
 		displaymsg->DisplayConstantStringName(STR_DOORBASH_FAIL, DMC_BG2XPGREEN, actor);
 		return;
@@ -401,6 +436,7 @@ void Door::TryBashLock(Actor *actor)
 
 	displaymsg->DisplayConstantStringName(STR_DOORBASH_DONE, DMC_LIGHTGREY, actor);
 	SetDoorLocked(false, true);
+	core->GetGameControl()->ResetTargetMode();
 	Flags|=DOOR_BROKEN;
 
 	//This is ok, bashdoor also sends the unlocked trigger
@@ -415,7 +451,7 @@ void Door::dump() const
 	buffer.appendFormatted( "Door Global ID: %d\n", GetGlobalID());
 	buffer.appendFormatted( "Position: %d.%d\n", Pos.x, Pos.y);
 	buffer.appendFormatted( "Door Open: %s\n", YESNO(IsOpen()));
-	buffer.appendFormatted( "Door Locked: %s\n", YESNO(Flags&DOOR_LOCKED));
+	buffer.appendFormatted( "Door Locked: %s	Difficulty: %d\n", YESNO(Flags&DOOR_LOCKED), LockDifficulty);
 	buffer.appendFormatted( "Door Trapped: %s	Difficulty: %d\n", YESNO(Trapped), TrapRemovalDiff);
 	if (Trapped) {
 		buffer.appendFormatted( "Trap Permanent: %s Detectable: %s\n", YESNO(Flags&DOOR_RESET), YESNO(Flags&DOOR_DETECTABLE) );
