@@ -52,6 +52,7 @@ namespace GemRB {
 // we start this at a non-zero value to make debugging easier
 static ieDword globalActorCounter = 10000;
 static bool startActive = false;
+static bool third = false;
 
 /***********************
  *  Scriptable Class   *
@@ -128,6 +129,7 @@ Scriptable::Scriptable(ScriptableType type)
 
 	memset( script_timers,0, sizeof(script_timers));
 	startActive = core->HasFeature(GF_START_ACTIVE);
+	third = core->HasFeature(GF_3ED_RULES);
 }
 
 Scriptable::~Scriptable(void)
@@ -226,9 +228,9 @@ void Scriptable::DisplayHeadText(const char* text)
 	if (overHeadText) {
 		core->FreeString( overHeadText );
 	}
-	overHeadText = (char *) text;
 	overHeadTextPos.empty();
 	if (text) {
+		overHeadText = strdup(text);
 		timeStartDisplaying = core->GetGame()->Ticks;
 		textDisplaying = 1;
 	}
@@ -633,7 +635,7 @@ void Scriptable::PartyRested()
 	AddTrigger(TriggerEntry(trigger_partyrested));
 }
 
-ieDword Scriptable::GetInternalFlag()
+ieDword Scriptable::GetInternalFlag() const
 {
 	return InternalFlags;
 }
@@ -861,31 +863,18 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 	if (spellnum!=0xffffffff) {
 		area->SeeSpellCast(this, spellnum);
 
-		// caster - Casts spellname : target OR
-		// caster - spellname : target (repeating spells)
-		Scriptable *target = NULL;
-		if (tgt) {
-			target = area->GetActorByGlobalID(tgt);
-			if (!target) {
-				target=core->GetGame()->GetActorByGlobalID(tgt);
+		// spellcasting feedback
+		if (third) {
+			// only display it for party friendly creatures - enemies require a successful spellcraft check
+			if (Type == ST_ACTOR) {
+				Actor *caster = ((Actor *) this);
+				if (caster->GetStat(IE_EA) <= EA_CONTROLLABLE) {
+					DisplaySpellCastMessage(tgt, spl);
+				}
 			}
+		} else {
+			DisplaySpellCastMessage(tgt, spl);
 		}
-		char* spell = core->GetString(spl->SpellName);
-		if (stricmp(spell, "") && Type == ST_ACTOR) {
-			char* msg = core->GetString(displaymsg->GetStringReference(STR_ACTION_CAST), 0);
-			char *tmp;
-			if (target) {
-				tmp = (char *) malloc(strlen(msg)+strlen(spell)+strlen(target->GetName(-1))+5);
-				sprintf(tmp, "%s %s : %s", msg, spell, target->GetName(-1));
-			} else {
-				tmp = (char *) malloc(strlen(spell)+strlen(GetName(-1))+4);
-				sprintf(tmp, "%s : %s", spell, GetName(-1));
-			}
-			displaymsg->DisplayStringName(tmp, DMC_WHITE, this);
-			core->FreeString(msg);
-			free(tmp);
-		}
-		core->FreeString(spell);
 	}
 	// only trigger the autopause when in combat or buffing gets very annoying
 	if (core->GetGame()->CombatCounter) {
@@ -895,6 +884,35 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 	gamedata->FreeSpell(spl, SpellResRef, false);
 }
 
+void Scriptable::DisplaySpellCastMessage(ieDword tgt, Spell *spl)
+{
+	// caster - Casts spellname : target OR
+	// caster - spellname : target (repeating spells)
+	Scriptable *target = NULL;
+	if (tgt) {
+		target = area->GetActorByGlobalID(tgt);
+		if (!target) {
+			target=core->GetGame()->GetActorByGlobalID(tgt);
+		}
+	}
+	char* spell = core->GetString(spl->SpellName);
+	if (stricmp(spell, "") && Type == ST_ACTOR) {
+		char* msg = core->GetString(displaymsg->GetStringReference(STR_ACTION_CAST), 0);
+		char *tmp;
+		if (target) {
+			tmp = (char *) malloc(strlen(msg)+strlen(spell)+strlen(target->GetName(-1))+5);
+			sprintf(tmp, "%s %s : %s", msg, spell, target->GetName(-1));
+		} else {
+			tmp = (char *) malloc(strlen(spell)+strlen(GetName(-1))+4);
+			sprintf(tmp, "%s : %s", spell, GetName(-1));
+		}
+		displaymsg->DisplayStringName(tmp, DMC_WHITE, this);
+		core->FreeString(msg);
+		free(tmp);
+	}
+	core->FreeString(spell);
+}
+
 void Scriptable::SendTriggerToAll(TriggerEntry entry)
 {
 	Actor** nearActors = area->GetAllActorsInRadius(Pos, GA_NO_DEAD, 15*10);
@@ -961,6 +979,10 @@ void Scriptable::CastSpellPointEnd(int level, int no_stance)
 		return;
 	}
 
+	if (caster && caster->PCStats) {
+		caster->PCStats->RegisterFavourite(SpellResRef, FAV_SPELL);
+	}
+
 	CreateProjectile(SpellResRef, 0, level, false);
 	//FIXME: this trigger affects actors whom the caster sees, not just the caster itself
 	// the original engine saves lasttrigger only in case of SpellCast, so we have to differentiate
@@ -1022,6 +1044,10 @@ void Scriptable::CastSpellEnd(int level, int no_stance)
 		return;
 	}
 
+	if (caster && caster->PCStats) {
+		caster->PCStats->RegisterFavourite(SpellResRef, FAV_SPELL);
+	}
+
 	//if the projectile doesn't need to follow the target, then use the target position
 	CreateProjectile(SpellResRef, LastTarget, level, GetSpellDistance(SpellResRef, this)==0xffffffff);
 	//FIXME: this trigger affects actors whom the caster sees, not just the caster itself
@@ -1050,7 +1076,7 @@ void Scriptable::CastSpellEnd(int level, int no_stance)
 }
 
 // check for input sanity and good casting conditions
-int Scriptable::CanCast(const ieResRef SpellResRef) {
+int Scriptable::CanCast(const ieResRef SpellResRef, bool verbose) {
 	Spell* spl = gamedata->GetSpell(SpellResRef);
 	if (!spl) {
 		SpellHeader = -1;
@@ -1096,24 +1122,32 @@ int Scriptable::CanCast(const ieResRef SpellResRef) {
 		// check for miscast magic and similar
 		ieDword roll = actor->LuckyRoll(1, 100, 0, 0);
 		bool failed = false;
+		ieDword chance = 0;
 		switch(spl->SpellType)
 		{
 		case IE_SPL_PRIEST:
-			if (actor->GetSpellFailure(false) >= roll) {
+			chance = actor->GetSpellFailure(false);
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		case IE_SPL_WIZARD:
-			if (actor->GetSpellFailure(true) >= roll) {
+			chance = actor->GetSpellFailure(true);
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		case IE_SPL_INNATE:
-			if (actor->Modified[IE_SPELLFAILUREINNATE] >= roll) {
+			chance = actor->Modified[IE_SPELLFAILUREINNATE];
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		}
+		if (verbose && chance && third) {
+			// ~Spell Failure check: Roll d100 %d vs. Spell failure chance %d~
+			displaymsg->DisplayRollStringName(40955, DMC_LIGHTGREY, actor, roll, chance);
+		}
 		if (failed) {
 			// TODO: display fizzling animation
 			displaymsg->DisplayConstantStringName(STR_MISCASTMAGIC, DMC_WHITE, this);
@@ -1124,6 +1158,50 @@ int Scriptable::CanCast(const ieResRef SpellResRef) {
 	return 1;
 }
 
+// checks if a party-friendly actor is nearby and if so, if it reckognizes the spell
+// this enemy just started casting
+void Scriptable::SpellcraftCheck(const Actor *caster, const ieResRef SpellResRef)
+{
+	if (!third || !caster || caster->GetStat(IE_EA) <= EA_CONTROLLABLE || !area) {
+		return;
+	}
+
+	Spell* spl = gamedata->GetSpell(SpellResRef);
+	assert(spl); // only a bad surge could make this fail and we want to catch it
+	int AdjustedSpellLevel = spl->SpellLevel + 15;
+	Actor **neighbours = area->GetAllActorsInRadius(caster->Pos, GA_NO_DEAD|GA_NO_ENEMY|GA_NO_SELF, 10*caster->GetBase(IE_VISUALRANGE));
+	Actor **poi = neighbours;
+	while (*poi) {
+		Actor *detective = *poi;
+		// disallow neutrals from helping the party
+		if (detective->GetStat(IE_EA) > EA_CONTROLLABLE) {
+			poi++;
+			continue;
+		}
+		if ((signed)detective->GetStat(IE_SPELLCRAFT) <= 0) {
+			poi++;
+			continue;
+		}
+
+		// ~Spellcraft check (d20 roll + Spellcraft level + int mod) %d vs. (spell level + 15)  = %d.   (Int mod = %d)~
+		int Spellcraft = core->Roll(1, 20, 0) + detective->GetStat(IE_SPELLCRAFT);
+		int IntMod = detective->GetAbilityBonus(IE_INT);
+
+		if ((Spellcraft + IntMod) > AdjustedSpellLevel) {
+			char tmpstr[100];
+			memset(tmpstr, 0, sizeof(tmpstr));
+			// eg. .:Casts Word of Recall:.
+			snprintf(tmpstr, sizeof(tmpstr), ".:%s %s:.", core->GetString(displaymsg->GetStringReference(STR_CASTS)), core->GetString(spl->SpellName));
+			DisplayHeadText(tmpstr);
+			displaymsg->DisplayRollStringName(39306, DMC_LIGHTGREY, detective, Spellcraft+IntMod, AdjustedSpellLevel, IntMod);
+			break;
+		}
+		poi++;
+	}
+	gamedata->FreeSpell(spl, SpellResRef, false);
+	free(neighbours);
+}
+
 //set target as point
 //if spell needs to be depleted, do it
 //if spell is illegal stop casting
@@ -1152,6 +1230,9 @@ int Scriptable::CastSpellPoint( const Point &target, bool deplete, bool instant,
 	if(!CheckWildSurge()) {
 		return -1;
 	}
+	if (!instant) {
+		SpellcraftCheck(actor, SpellResRef);
+	}
 	return SpellCast(instant);
 }
 
@@ -1190,6 +1271,9 @@ int Scriptable::CastSpell( Scriptable* target, bool deplete, bool instant, bool
 		return -1;
 	}
 
+	if (!instant) {
+		SpellcraftCheck(actor, SpellResRef);
+	}
 	return SpellCast(instant);
 }
 
@@ -1301,8 +1385,7 @@ int Scriptable::CheckWildSurge()
 
 				// lookup the spell in the "check" row of wildmag.2da
 				ieResRef surgeSpellRef;
-				memset(surgeSpellRef, 0, sizeof(surgeSpellRef));
-				strncpy(surgeSpellRef, core->SurgeSpells[check-1].spell, 8);
+				CopyResRef(surgeSpellRef, core->SurgeSpells[check-1].spell);
 
 				if (!gamedata->Exists(surgeSpellRef, IE_SPL_CLASS_ID)) {
 					// handle the hardcoded cases - they'll also fail here
@@ -1314,7 +1397,7 @@ int Scriptable::CheckWildSurge()
 				} else {
 					// finally change the spell
 					// the hardcoded bunch does it on its own when needed
-					strncpy(SpellResRef, surgeSpellRef, 8);
+					CopyResRef(SpellResRef, surgeSpellRef);
 				}
 			}
 		}
@@ -1383,17 +1466,17 @@ bool Scriptable::HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor
 			for (i=0; i<count; i++) {
 				if (target) {
 					caster->CastSpell(target, false, true);
-					strncpy(newspl, SpellResRef, 8);
+					CopyResRef(newspl, SpellResRef);
 					caster->WMLevelMod = tmp3;
 					caster->CastSpellEnd(level, 1);
 				} else {
 					caster->CastSpellPoint(targetpos, false, true);
-					strncpy(newspl, SpellResRef, 8);
+					CopyResRef(newspl, SpellResRef);
 					caster->WMLevelMod = tmp3;
 					caster->CastSpellPointEnd(level, 1);
 				}
 				// reset the ref, since CastSpell*End destroyed it
-				strncpy(SpellResRef, newspl, 8);
+				CopyResRef(SpellResRef, newspl);
 			}
 			caster->Modified[IE_FORCESURGE] = tmp;
 			caster->Modified[IE_SURGEMOD] = tmp2;
@@ -1420,7 +1503,7 @@ bool Scriptable::HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor
 				int id = core->Roll(1, spellCount, -1);
 				CREKnownSpell *ck = caster->spellbook.GetKnownSpell(i, lvl, id);
 				if (ck) {
-					strncpy(SpellResRef, ck->SpellResRef, 8);
+					CopyResRef(SpellResRef, ck->SpellResRef);
 					break;
 				}
 			}
@@ -1756,7 +1839,20 @@ void Highlightable::DetectTrap(int skill, ieDword actorID)
 	if (!CanDetectTrap()) return;
 	if (!Scripts[0]) return;
 	if ((skill>=100) && (skill!=256) ) skill = 100;
-	if (skill/2+core->Roll(1,skill/2,0)>TrapDetectionDiff) {
+	int check = 0;
+	if (third) {
+		//~Search (detect traps) check. Search skill %d vs. trap's difficulty %d (searcher's %d INT bonus).~
+		Actor *detective = core->GetGame()->GetActorByGlobalID(actorID);
+		int bonus = 0;
+		if (detective) {
+			bonus = detective->GetAbilityBonus(IE_INT);
+			displaymsg->DisplayRollStringName(39303, DMC_LIGHTGREY, detective, skill-bonus, TrapDetectionDiff, bonus);
+		}
+		check = (skill + bonus)*7;
+	} else {
+		check = skill/2 + core->Roll(1, skill/2, 0);
+	}
+	if (check > TrapDetectionDiff) {
 		SetTrapDetected(1); //probably could be set to the player #?
 		AddTrigger(TriggerEntry(trigger_detected, actorID));
 	}
@@ -1886,13 +1982,12 @@ void Movable::SetAttackMoveChances(ieWord *amc)
 //this could be used for WingBuffet as well
 void Movable::MoveLine(int steps, int Pass, ieDword orient)
 {
-	//remove previous path
-	ClearPath();
-	if (!steps)
+	if (path || !steps) {
 		return;
+	}
 	Point p = Pos;
 	p.x/=16;
-	p.y/=14;
+	p.y/=12;
 	path = area->GetLine( p, steps, orient, Pass );
 }
 
@@ -2114,9 +2209,9 @@ void Movable::RandomWalk(bool can_stop, bool run)
 	//p.x+=x;
 	//p.y+=(int) sqrt(100-x*x);
 
-	//selecting points in a square around actor
-	p.x+=core->Roll(1,50,-25);
-	p.y+=core->Roll(1,50,-25);
+	//selecting points in a square around actor (-24 to +24)
+	p.x+=core->Roll(1,49,-25);
+	p.y+=core->Roll(1,49,-25);
 	//the 5th parameter is controlling the orientation of the actor
 	//0 - back away, 1 - face direction
 	path = area->RunAway( Pos, p, size, 50, 1 );
