@@ -34,7 +34,7 @@
 
 #include "ie_types.h"
 
-#define VERSION_GEMRB "0.7.2maemo1"
+#define VERSION_GEMRB "0.8.0maemo1"
 
 #define GEMRB_STRING "GemRB v" VERSION_GEMRB
 
@@ -156,9 +156,15 @@ namespace GemRB {
 #define  GF_RARE_ACTION_VB              66 //pst
 #define  GF_NO_UNDROPPABLE              67 //iwd,how
 #define  GF_START_ACTIVE                68 //bg1
+#define  GF_CLEAR_UNUSED_AREA           69 //bg1
+#define  GF_IMPLICIT_AREAANIM_BACKGROUND    70 //idw,how,iwd2
+#define  GF_HEAL_ON_100PLUS             71 //bg1, bg2, pst
+#define  GF_IN_PARTY_ALLOWS_DEAD	72 //all except bg2
+#define  GF_ZERO_TIMER_IS_VALID         73 // how, not bg2, other unknown
+#define  GF_SKIPUPDATE_HACK             74 // how, not bg2
 
 //update this or bad things can happen
-#define GF_COUNT 69
+#define GF_COUNT 75
 
 //the number of item usage fields (used in CREItem and STOItem)
 #define CHARGE_COUNTERS  3
@@ -186,6 +192,8 @@ GEM_EXPORT unsigned int PersonalDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT unsigned int SquaredPersonalDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT unsigned int SquaredMapDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT int EARelation(Scriptable *a, Actor *b);
+GEM_EXPORT void CopyResRef(ieResRef d, const ieResRef s);
+
 
 #ifndef WIN32
 inline unsigned long GetTickCount()
